# Copyright (C) 2011 W. Trevor King <wking@drexel.edu>
#
# This file is part of h5config.
#
# h5config is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# h5config is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with h5config.  If not, see <http://www.gnu.org/licenses/>.


class Storage (object):
    "A storage bakend for loading and saving `Config` instances"
    def load(self, config, merge=False, **kwargs):
        if merge:
            self.clear()
        self._load(config=config, **kwargs)
        config.set_storage(storage=self)

    def _load(self, config, **kwargs):
        raise NotImplementedError()

    def save(self, config, merge=False, **kwargs):
        if merge:
            self.clear()
        self._save(config=config, **kwargs)
        config._storage = self

    def _save(self, config, **kwargs):
        raise NotImplementedError()

    def clear(self):
        raise NotImplementedError()


class FileStorage (Storage):
    "`Config` storage backend by a single file"
    extension = None

    def __init__(self, filename=None):
        self._filename = filename
